#include "level.h"

#include <fstream>
#include <cstring>

void Level::prepareCellInstances()
{
    default_cells[PASSABLE_CELL] = new PassableCell();
    default_cells[WATER_CELL] = new WaterCell();
    default_cells[WALL_CELL] = new WallCell();
    default_cells[CHARGE_CELL] = new ChargeCell();
    default_cells[EXIT_CELL] = new ExitCell();
    default_cells[TELEPORT_CELL] = new TeleportCell();
    default_cells[TRIGGER_CELL] = new TriggerCell();
}

void Level::readMap(std::ifstream &file)
{
    int i;
    for (coordinate j = 0; j < map.size(); ++j)
    {
        for (coordinate k = 0; k < map[j].size(); ++k)
        {
            file >> i;
            map[j][k] = default_cells[i]->getDefaultInstance();
            map[j][k]->setPosition(j, k);
        }
    }
}

Level::Level(const std::string &map_file)
{
    prepareCellInstances();

    std::ifstream file;
    file.open(map_file);

    std::string cur_line;
    while (getline(file, cur_line))
    {
        // need fix; see std::string.compare
        if (cur_line.compare(0, 4, "size") == 0)
        {
            file >> level_rows >> level_cols;
            map.resize(level_rows);
            for (Row &row : map)
                row.resize(level_cols);
        }
        else if (cur_line.compare(0, 3, "map") == 0)
        {
            readMap(file);
        }
        else if (cur_line.compare(0, 8, "teleport") == 0)
        {
            coordinate src_row, src_col;
            coordinate dest_row, dest_col;

            file >> src_row >> src_col >> dest_row >> dest_col;
            // reinterpret_cast<TeleportCell *>(map[src_row][src_col])->setDestination(dest_row, dest_col);
        }
    }
}

Level::~Level()
{
    for (Cell *cell : default_cells)
        delete cell;

    for (Row &row : map)
        for (Cell *cell : row)
            delete cell;
}

size_t Level::rows() const
{
    return level_rows;
}

size_t Level::cols() const
{
    return level_cols;
}

void Level::placeBridge(coordinate row, coordinate col)
{
    Cell *buf = map[row][col];
    map[row][col] = new PassableCell(row, col, sf::Color::Black);
    delete buf;
}

void Level::removeCharge(coordinate row, coordinate col)
{
    Cell *buf = map[row][col];
    map[row][col] = new PassableCell(row, col, color_ground);
    delete buf;
}

Map& Level::mapArray()
{
    return map;
}

sf::Color Level::defaultGroundColor()
{
    return color_ground;
}

void Level::setDefaultGroundColor(const sf::Color &color)
{
    color_ground = color;
}
